@@ -1,7 +1,7 @@
 /*
  * player.cpp
  *
- * Copyright 2002-2014 Vesa Halttunen
+ * Copyright 2002-2016 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -264,9 +264,8 @@ void Player::stopMuted()
             QSharedPointer<TrackStatus> trackStatus = trackStatuses[track];
             if (trackStatus->note != -1) {
                 midi_->output(trackStatus->midiInterface)->noteOff(trackStatus->midiChannel, trackStatus->note, 127);
-                trackStatus->note = -1;
             }
-            trackStatus->baseNote = -1;
+            trackStatus->reset();
         }
     }
 }
@@ -281,13 +280,8 @@ void Player::stopNotes()
         QSharedPointer<TrackStatus> trackStatus = trackStatuses[track];
         if (trackStatus->note != -1) {
             midi_->output(trackStatus->midiInterface)->noteOff(trackStatus->midiChannel, trackStatus->note, 127);
-
-            trackStatus->midiChannel = -1;
-            trackStatus->note = -1;
-            trackStatus->volume = -1;
-            trackStatus->hold = -1;
         }
-        trackStatus->baseNote = -1;
+        trackStatus->reset();
     }
 }
 
@@ -348,7 +342,7 @@ void Player::handleCommand(QSharedPointer<TrackStatus> trackStatus, unsigned cha
 
     switch (command) {
     case CommandPitchWheel:
-        // Program change can be sent if the MIDI channel is known
+        // Pitch wheel can be set if the MIDI channel is known
         if (midiChannel != -1) {
             if (value < 0x80) {
                 if (tick == 0) {
@@ -368,7 +362,7 @@ void Player::handleCommand(QSharedPointer<TrackStatus> trackStatus, unsigned cha
         }
         break;
     case CommandProgramChange:
-        // Pitch wheel can be set if the MIDI channel is known
+        // Program change can be sent if the MIDI channel is known
         if (midiChannel != -1 && tick == 0) {
             output->programChange(midiChannel, value & 0x7f);
         }
@@ -506,6 +500,7 @@ void Player::run()
 {
     ExternalSync prevsyncMode = syncMode;
     unsigned int oldTime = (unsigned int)-1;
+    unsigned int oldLine = line_;
 
     tick = 0;
     ticksSoFar = 0;
@@ -516,7 +511,7 @@ void Player::run()
 
     while (true) {
         bool looped = false;
-        unsigned int oldLine = line_;
+        oldLine = line_;
 
         // Lock
         mutex.lock();
@@ -547,6 +542,7 @@ void Player::run()
         }
 
         Block *block = song->block(block_);
+        int commandPages = block->commandPages();
 
         // Send MIDI sync if requested
         if (song->sendSync()) {
@@ -591,7 +587,7 @@ void Player::run()
                 }
 
                 if (note != 0) {
-                    for (int commandPage = 0; commandPage < block->commandPages(); commandPage++) {
+                    for (int commandPage = 0; commandPage < commandPages; commandPage++) {
                         unsigned char command = block->command(line_, track, commandPage);
                         unsigned char value = block->commandValue(line_, track, commandPage);
 
@@ -669,7 +665,7 @@ void Player::run()
                 }
 
                 // Handle commands on all command pages
-                for (int commandPage = 0; commandPage < block->commandPages(); commandPage++) {
+                for (int commandPage = 0; commandPage < commandPages; commandPage++) {
                     unsigned char command = block->command(line_, track, commandPage);
                     unsigned char value = block->commandValue(line_, track, commandPage);
                     handleCommand(trackStatus, note, instrument, command, value, &volume, &delay, &hold);
@@ -850,6 +846,10 @@ void Player::run()
     // The mutex is locked if the thread was killed and loop broken
     song->unlock();
     mutex.unlock();
+
+    if (line_ != oldLine) {
+        emit lineChanged(line_);
+    }
 }
 
 void Player::playWithoutScheduling()
@@ -869,6 +869,7 @@ void Player::play(Mode mode, bool cont)
     stop();
 
     Mode oldMode = mode_;
+    int oldLine = line_;
     mode_ = mode;
     tick = 0;
     ticksSoFar = 0;
@@ -891,6 +892,10 @@ void Player::play(Mode mode, bool cont)
         break;
     }
 
+    if (line_ != oldLine) {
+        emit lineChanged(line_);
+    }
+
     // Get the starting time
     resetTime(!cont);
 
@@ -949,26 +954,18 @@ void Player::continueBlock()
     play(ModePlayBlock, true);
 }
 
-void Player::trackStatusCreate()
+void Player::trackStatusCreate(bool recreateAll)
 {
     int maxTracks = song != NULL ? song->maxTracks() : 0;
 
     // Free the extraneous status structures
-    while (trackStatuses.count() > maxTracks) {
+    while (trackStatuses.count() > (recreateAll ? 0 : maxTracks)) {
         trackStatuses.removeLast();
     }
 
     // Set new tracks to -1
     while (trackStatuses.count() < maxTracks) {
-        TrackStatus *status = new TrackStatus;
-        status->instrument = -1;
-        status->previousCommand = 0;
-        status->note = -1;
-        status->midiChannel = -1;
-        status->midiInterface = -1;
-        status->volume = -1;
-        status->hold = -1;
-        trackStatuses.append(QSharedPointer<TrackStatus>(status));
+        trackStatuses.append(QSharedPointer<TrackStatus>(new TrackStatus));
     }
 }
 
@@ -989,7 +986,7 @@ void Player::setSong(const QString &path)
         QByteArray header = file.read(4);
         if (header.length() == 4) {
             const char *data = header.data();
-            if (data[0] == (ID_MMD2 >> 24) && data[1] == ((ID_MMD2 >> 16) & 0xff) && data[2] == ((ID_MMD2 >> 8) & 0xff) && data[3] == (ID_MMD2 & 0xff)) {
+            if (data[0] == (ID_MMD2 >> 24) && data[1] == ((ID_MMD2 >> 16) & 0xff) && data[2] == ((ID_MMD2 >> 8) & 0xff) && (data[3] == (ID_MMD0 & 0xff) || data[3] == (ID_MMD1 & 0xff) || data[3] == (ID_MMD2 & 0xff))) {
                 song = mmd2ToSong(MMD2_load(path.toUtf8().constData()));
             }
         }
@@ -1013,7 +1010,7 @@ void Player::init()
     remapMidiOutputs();
 
     // Recreate the track status array
-    trackStatusCreate();
+    trackStatusCreate(true);
     connect(this->song, SIGNAL(maxTracksChanged(uint)), this, SLOT(trackStatusCreate()));
 
     // Check solo status
@@ -1286,3 +1283,19 @@ MIDI *Player::midi() const
 {
     return midi_;
 }
+
+Player::TrackStatus::TrackStatus()
+{
+    reset();
+}
+
+void Player::TrackStatus::reset()
+{
+    instrument = -1;
+    previousCommand = 0;
+    note = -1;
+    midiChannel = -1;
+    midiInterface = -1;
+    volume = -1;
+    hold = -1;
+}
